clear, clc

%Ingresamos datos.
Iy = 6928.93; %Momento de Inercia respecto a y. [kgm^2]
m = 2288.3104; %Masa [kg]
mac = 1.5875; %Cuerma Media Aerodinamica [m]
S = 23.23; %Superficie de Referencia (alar) [m^2]
Clo = 0.2; %Sustentacin a incidencia nula
Cl_alfa_deg = 0.09155; %Pendiente de sustentacion [1/]
Cl_alfa_rad = 5.2458; %Pendiente de sustentacion [1/rad]
Cdo = 0.0203; %Trmino independiente de la polar
x1 = -0.0141; %Trmino lineal de la polar
K = 0.054; %Termino cuadrtico de la polar

%Ingresamos funciones a usar.
densidad = @(x) 1.225*(1-22.558e-6 * x)^4.2561;
Cl = @(x) 0.2 + 0.091557 * x;
Cd = @(x) 0.0203 - 0.0141 * x + 0.054 * x^2;

%%
%                                              DESPEGUE
%Ingresamos datos para este caso.
Vo = 35; %Velocidad de Despegue [m/s]
alfa = 0.2864; %ngulo de Ataque [] = 0.005 rad
tita = 5.9587; %Pitch angle [] o 0.104 rad
h = 0; %Altura [m]

%Datos de derivadas del libro.
CLq = 2.94627;
CMq = -15.51507;

%Datos de trabajos anteriores.
grad_downwash = 0.4701;
Cm_alfa = -0.89381; %Adems, Cm_alfa = -0.0156 [1/]
Kn_pot = 0.109678; %Margen esttico mando fijo con potencia
Kn_nopot = 0.152768; %Margen esttico mando fijo sin potencia
l_T = 6.011; %Distancia MAC del empenaje al cg. [m]

%Calculamos datos para adimesionalizar.
mu = 2 * m /(densidad(h) * S * mac); %Densidad Longitudinal Relativa
sigma = 2 * m / (densidad(h) * Vo * S);

%Calculamos datos adimensionalizados.
i_y = Iy / (m*mac^2);

%Coeficientes necesarios para calcular las derivadas.
CL = Cl(alfa);
CD = Cd(CL);

Cdf_v = sym('Cdo + x1 * 2 * W /(rho * V^2 * S) + K * 4 * W^2 / (rho^2 * V^4 * S^2)'); %Para calcular    dCD/dV !!
dCdf_v_dv = diff(Cdf_v, 'V');
dCD_dv = double(subs(dCdf_v_dv, {'x1','K','rho','W','S','V'}, {x1,K,densidad(h),m,S,Vo}));

Cdf_alfa = sym('Cdo + x1 * (Clo + Cl_alfa * alfa) + K * (Clo + Cl_alfa * alfa)^2'); %Para calcular  dCD/dALFA !! 
dCdf_alfa_dalfa = diff(Cdf_alfa, 'alfa');
dCD_dalfa = double(subs(dCdf_alfa_dalfa, {'x1','K','Clo','Cl_alfa','alfa'}, {x1,K,Clo,Cl_alfa_rad,alfa})); %POSIBLE RAIDERS EN LA PENDIENTE (ponerla en deg)

Clf_v = sym('2 * W / (rho * V^2 * S)'); %Para calcular  dCL/dV !!
dClf_v_dv = diff(Clf_v, 'V');
dCL_dv = double(subs(dClf_v_dv, {'W','V','rho','S'}, {m,Vo,densidad(h),S}));

Cmf_v = sym('Cmo - 0.3066 * 2 * W /(rho * V^2 * S)');
dCmf_v_dv = diff(Cmf_v, 'V');
dCM_dv = double(subs(dCmf_v_dv, {'W','rho','V','S'},{m,densidad(h),Vo,S}));

%Derivadas.

Xu = - 2 * CD - Vo * dCD_dv;
Xw = CL - dCD_dalfa; %VER RAIDERS, SI PONGO PENDIENTE EN 1/DEG ME DA 0.22
Xq = 0; %Por apndice 8 Cook
X_dotw = 0; %Ya que Xq = 0

Zu = - 2 * CL - Vo * dCL_dv;
Zw = - CD - Cl_alfa_rad;
Zq = - CLq / 2;
Z_dotw = Zq * grad_downwash;

Mu = Vo * dCM_dv;
Mw = -Cl_alfa_rad * Kn_nopot;
Mq = Zq * l_T / mac;
M_dotw = Mq * grad_downwash;

%Definimos las matrices.

M = [1  -X_dotw/mu      0              0;
     0 1-Z_dotw/mu      0              0;
     0  -M_dotw/mu   i_y/mu            0;
     0       0          0              1];
 
A_prima =  [Xu    Xw    Xq/mu     -CL*cos(tita);
            Zu    Zw   1+Zq/mu    -CL*sin(tita);
            Mu    Mw    Mq/mu           0      ;
            0     0       1             0      ];
  
Matriz_A = M\A_prima; %Matlab recomienda esto para que sea ms rapido inv(M)*A_prima = M\A_prima. La barra va hacia la que no es inversa

%Encontramos valores propios, vectores propios y coeficientes del polinomio caracterstico.

polinomio_caracteristico = poly(Matriz_A); %Obtiene el polinomio caracteristico

A = polinomio_caracteristico(1,1); %Coeficientes del polinomio caracterstico
B = polinomio_caracteristico(1,2);
C = polinomio_caracteristico(1,3);
D = polinomio_caracteristico(1,4);
E = polinomio_caracteristico(1,5);

a2 = 0;  %Calculo de los coeficientes a1, b1, a2, b2 para armar factorizacin de polinomio caracterstico
b2 = 0;
for k = 1:3 
    a1 = B - a2;
    b1 = C - a1 * a2  - b2;
    b2 = E/b1;
    a2 = (D - a1 * b2)/b1;
end

fprintf('El polinomio factorizado es = (y^2 + %4.2f * y + %4.2f)*(y^2 + %4.2f * y + %4.2f)\n', a1, b1, a2, b2)

[vectores, valores] = eig(Matriz_A); %Obtenemos valores y vectores propios.

vector_propio_1 = vectores(:,1); %Vectores propios
vector_propio_2 = vectores(:,2);
vector_propio_3 = vectores(:,3);
vector_propio_4 = vectores(:,4);

valor_propio_1 = valores(1,1); %Valores propios
valor_propio_2 = valores(2,2);
valor_propio_3 = valores(3,3);
valor_propio_4 = valores(4,4);

x_inicial = [0.1; 0.1; 1; 0];
z_inicial = vectores\x_inicial;

lambdas = [valor_propio_1; valor_propio_2; valor_propio_3; valor_propio_4] / sigma

%Para el corto perodo.

t_short = linspace(0,5,1000);

u_short    = (z_inicial(1,1) * vector_propio_1(1,1) * exp(lambdas(1,1)*t_short) + z_inicial(2,1) * vector_propio_2(1,1) * exp(lambdas(2,1)*t_short))*Vo;
w_short    = (z_inicial(1,1) * vector_propio_1(2,1) * exp(lambdas(1,1)*t_short) + z_inicial(2,1) * vector_propio_2(2,1) * exp(lambdas(2,1)*t_short))*Vo;
q_short    = (z_inicial(1,1) * vector_propio_1(3,1) * exp(lambdas(1,1)*t_short) + z_inicial(2,1) * vector_propio_2(3,1) * exp(lambdas(2,1)*t_short))*sigma;
tita_short = (z_inicial(1,1) * vector_propio_1(4,1) * exp(lambdas(1,1)*t_short) + z_inicial(2,1) * vector_propio_2(4,1) * exp(lambdas(2,1)*t_short));

figure(1)
subplot(2,2,1)
plot(t_short, u_short), title('Corto Periodo, u(t)')

subplot(2,2,2)
plot(t_short, w_short), title('Corto Periodo, w(t)')

subplot(2,2,3)
plot(t_short, q_short), title('Corto Periodo, q(t)')

subplot(2,2,4)
plot(t_short, tita_short), title('Corto Periodo, \theta(t)')

%Para el fugoide.

t_phugoid = linspace(0,300,1000);
 
u_phugoid    = (z_inicial(3,1) * vector_propio_3(1,1) * exp(lambdas(3,1)*t_phugoid) + z_inicial(4,1) * vector_propio_4(1,1) * exp(lambdas(4,1)*t_phugoid))*Vo;
w_phugoid    = (z_inicial(3,1) * vector_propio_3(2,1) * exp(lambdas(3,1)*t_phugoid) + z_inicial(4,1) * vector_propio_4(2,1) * exp(lambdas(4,1)*t_phugoid))*Vo;
q_phugoid    = (z_inicial(3,1) * vector_propio_3(3,1) * exp(lambdas(3,1)*t_phugoid) + z_inicial(4,1) * vector_propio_4(3,1) * exp(lambdas(4,1)*t_phugoid))*sigma;
tita_phugoid = (z_inicial(3,1) * vector_propio_3(4,1) * exp(lambdas(3,1)*t_phugoid) + z_inicial(4,1) * vector_propio_4(4,1) * exp(lambdas(4,1)*t_phugoid));

figure(2)
subplot(2,2,1)
plot(t_phugoid, u_phugoid), title('Fugoide, u(t)')
 
subplot(2,2,2)
plot(t_phugoid, w_phugoid), title('Fugoide, w(t)')
 
subplot(2,2,3)
plot(t_phugoid, q_phugoid), title('Fugoide, q(t)')
 
subplot(2,2,4)
plot(t_phugoid, tita_phugoid), title('Fugoide, \theta(t)')
 


